package mi.hdm.typeAdapters;

import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import mi.hdm.recipes.Category;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public class CategoryTypeAdapter extends TypeAdapter<Category> {
    private final static Logger log = LogManager.getLogger(CategoryTypeAdapter.class);

    @Override
    public void write(JsonWriter jsonWriter, Category category) throws IOException {
        jsonWriter.beginObject();
        jsonWriter.name("name").value(category.getName());
        jsonWriter.name("colourCode").value(category.getColourCode());
        jsonWriter.name("code").value(category.getCode());
        jsonWriter.endObject();
    }

    @Override
    public Category read(JsonReader jsonReader) throws IOException {
        String name = null;
        String colourCode = null;
        Integer code = null;

        jsonReader.beginObject();
        while (jsonReader.hasNext()) {
            String propertyName = jsonReader.nextName();
            switch (propertyName) {
                case "name" -> name = jsonReader.nextString();
                case "colourCode" -> colourCode = jsonReader.nextString();
                case "code" -> code = jsonReader.nextInt();
                default -> jsonReader.skipValue();
            }
        }
        jsonReader.endObject();

        if (name == null || colourCode == null || code == null) {
            String msg = "Null value: invalid or missing value in JSON object for a category";
            log.error(msg);
            throw new IOException(msg);
        }

        return new Category(name, colourCode, code);
    }
}
