package mi.hdm.typeAdapters;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import mi.hdm.recipes.NutritionTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public class NutritionTableTypeAdapter extends TypeAdapter<NutritionTable> {
    private final static Logger log = LogManager.getLogger(NutritionTableTypeAdapter.class);

    @Override
    public void write(JsonWriter jsonWriter, NutritionTable nutritionTable) throws IOException {
        jsonWriter.beginObject();
        jsonWriter.name("values").value(new Gson().toJson(nutritionTable.getTable()));
        jsonWriter.endObject();
    }

    @Override
    public NutritionTable read(JsonReader jsonReader) throws IOException {
        Double calories = null, carbs = null, fat = null, proteins = null, fibers = null, salt = null;
        jsonReader.beginObject();
        if (jsonReader.hasNext()) {
            String propertyName = jsonReader.nextName();
            if (propertyName.equals("values")) {
                String valuesJson = jsonReader.nextString();
                JsonObject valuesObject = JsonParser.parseString(valuesJson).getAsJsonObject();
                calories = valuesObject.get("CALORIES").getAsDouble();
                carbs = valuesObject.get("CARBS").getAsDouble();
                fat = valuesObject.get("FAT").getAsDouble();
                proteins = valuesObject.get("PROTEINS").getAsDouble();
                fibers = valuesObject.get("FIBERS").getAsDouble();
                salt = valuesObject.get("SALT").getAsDouble();
            } else {
                jsonReader.skipValue();
            }
        }
        jsonReader.endObject();
        try {
            return new NutritionTable(calories, carbs, fat, proteins, fibers, salt);
        } catch (NullPointerException e) {
            log.error("IO Exception: invalid or missing values in JSON object for ingredient.");
            throw new IOException("null pointer exception - invalid or missing values in JSON");
        }
    }
}
