package mi.hdm.typeAdapters;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.TypeAdapter;
import com.google.gson.reflect.TypeToken;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import mi.hdm.recipes.NutritionTable;
import mi.hdm.recipes.Recipe;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

public class RecipeTypeAdapter extends TypeAdapter<Recipe> {
    private final static Logger log = LogManager.getLogger(RecipeTypeAdapter.class);

    @Override
    public void write(JsonWriter writer, Recipe recipe) throws IOException {
        writer.beginObject();
        writer.name("name").value(recipe.getName());
        writer.name("nutritionTable").jsonValue(new GsonBuilder().registerTypeAdapter(NutritionTable.class, new NutritionTableTypeAdapter()).create().toJson(recipe.getNutritionTable()));
        writer.name("code").value(recipe.getUniqueCode());

        writer.name("ingredients").jsonValue(new Gson().toJson(recipe.getIngredients()));

        writer.name("description").value(recipe.getDescription());
        writer.name("preparation").jsonValue(new Gson().toJson(recipe.getPreparation()));

        writer.name("categories").jsonValue(new Gson().toJson(recipe.getCategoryCodes()));

        writer.name("preparationTimeMins").value(recipe.getPreparationTimeMins());
        writer.name("creationTime").value(recipe.getCreationTime().toString());
        writer.endObject();
        log.debug("Wrote recipe '{}' to JSON.", recipe.getName());
    }

    @Override
    public Recipe read(JsonReader reader) throws IOException {
        String name = null;
        Map<String, Integer> ingredients = null;
        String description = null;
        List<String> preparation = null;
        List<Integer> categories = null;
        Integer preparationTimeMins = null;
        NutritionTable nutritionTable = null;
        LocalDateTime creationTime = null;
        String code = null;

        reader.beginObject();
        while (reader.hasNext()) {
            String propertyName = reader.nextName();
            switch (propertyName) {
                case "name":
                    name = reader.nextString();
                    break;
                case "ingredients":
                    ingredients = new Gson().fromJson(reader, new TypeToken<Map<String, Integer>>() {
                    }.getType());
                    break;
                case "description":
                    description = reader.nextString();
                    break;
                case "preparation":
                    preparation = new Gson().fromJson(reader, new TypeToken<List<String>>() {
                    }.getType());
                    break;
                case "categories":
                    categories = new Gson().fromJson(reader, new TypeToken<List<Integer>>() {
                    }.getType());
                    break;
                case "preparationTimeMins":
                    preparationTimeMins = reader.nextInt();
                    break;
                case "nutritionTable":
                    nutritionTable = new GsonBuilder().registerTypeAdapter(NutritionTable.class, new NutritionTableTypeAdapter()).create().fromJson(reader, NutritionTable.class);
                    break;
                case "creationTime":
                    creationTime = LocalDateTime.parse(reader.nextString());
                    break;
                case "code":
                    code = reader.nextString();
                    break;
                default:
                    reader.skipValue();
                    break;
            }
        }
        reader.endObject();

        log.debug("Read recipe '{}' from JSON.", name);

        return new Recipe(name, ingredients, description, preparation, categories, preparationTimeMins, nutritionTable, creationTime, code);
    }
}
